# V820.tcl---CAEN V820 Scaler module control frame


set frame V820
set frametitle "CAEN V820 Scaler Setup & Monitor"
set icontitle "V820"

set modulename v820
set moduletype  "CAEN V820"


set NUM 32

set maxmodule [inform vmecontrol inquire [set modulename]count]
if {[string range $maxmodule 0 4] == "can't"} then {set maxmodule 1}
set modulelist {}
for {set i 1} {$i <= $maxmodule} {incr i} {lappend modulelist $i}
set maxcounter [expr $maxmodule * $NUM]

set module [lindex $modulelist 0]

set rates 0
set rateperiod 10
set elapseperiod 10400

for {set c 0} {$c < $maxcounter} {incr c} {set data($c) 0; set data1($c) 0; set data2($c) 0; set rate($c) 0}
set rateid 0


loadsource common.vme

proc click_rates {v} {
   global rates rateperiod rateid data data1 data2 maxcounter
   set rates $v
   :modify RATES :value $rates
   if {$rates == 0} then {
       after cancel $rateid
       set rateid 0
       display_rates
   } else {
      if {$rateid != 0} then {return}
      display_counters
      for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data($c)}
      set rateid [after [expr $rateperiod * 1000] rates_update]
   }
}

proc rates_update {} {
   global rates rateperiod rateid data data1 data2 maxcounter
   if {$rates == 0} then {return}

   if {[winfo ismapped .]} then {display_counters}
   for {set c 0} {$c < $maxcounter} {incr c} {set data1($c) $data($c)}
   if {[winfo ismapped .]} then {display_rates}
   for {set c 0} {$c < $maxcounter} {incr c} {set data2($c) $data1($c)}
   set rateid [after [expr $rateperiod * 1000] rates_update]
}

proc click_clear {} {
   global modulename

#  clear all counters
     write-vme-registers $modulename#?.Clear 0
}

proc do_trig {} {
   global modulename

#  increment all counters
     write-vme-registers $modulename#?.Trig 0
}

proc display_counters {} {
   global NUM modulename module data maxcounter EGreport

   set EGreport -1
   set-busy
   
   for {set c 0} {$c < $maxcounter} {incr c} {
       set module [expr ($c / $NUM) + 1]
       set i [expr $c - (($module - 1) * $NUM)]
#    read it
       set data($c) [read-vme-register $modulename#$module.Counter$i]
       :modify C[set c]DATA  :value $data($c)
   }
   clear-busy
}

proc display_rates {} {
   global rate data1 data2 rates elapseperiod maxcounter

   clear-footer
   for {set c 0} {$c < $maxcounter} {incr c} {
       if {$rates == 0} then {
          set data 0
       } else {
          set data [expr (($data1($c) - $data2($c)) * 1000) / $elapseperiod]
       }
       set rate($c) $data
       :modify C[set c]RATE  :value $rate($c)
   }
   set-footer "Rates last updated [date]"
}


proc click_redisplay {} {display_counters}    

proc click arg {
   global debug actionlist
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

      RATES  {click_rates $v}

      default           {
           set p "click_[lowercase $w]"
           if {[info procs $p] == "$p"} then {$p} else {insert-debug "$arg"}
      }
   }
}

proc make-frame {} {
  global env maxcounter rates

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10

      :message M1 :label "Counter Values"

      for {set i 0} {$i < $maxcounter} {incr i} {
        if {[expr $i/8*8] == $i} then {:next-row T}
        :text C[set i]DATA :label $i: :valuex 3c :w 10 :value 0 :deaf T
      }

      :next-row T
      :message M2 :label "Rate Values"

      for {set i 0} {$i < $maxcounter} {incr i} {
        if {[expr $i/8*8] == $i} then {:next-row T}
        :text C[set i]RATE :label $i: :valuex 3c :w 10 :value 0 :deaf T
      }

   :panel CLICK :layout T :padx 10
      :layout H; :col-gap 10

      :button REDISPLAY  :label "Update Counters" 
      :button CLEAR      :label "Clear All Counters"
      :hskip 50
      :checkbox RATES    :label "Enable Rates" :value $rates

   :show T
}
